selfDir_3d1bc249 := $(dir $(lastword $(MAKEFILE_LIST)))

# Make
subMake = $(MAKE) $(@F) --directory=$(@D)
+_ = $(+:_%=)
addBeforeExt = $(basename $(2))$(1)$(suffix $(2))

# Commands
CP = cp -p
DIFF = diff --unified=2

tablesSort := plots organisms stems
vegxMaps := $(wildcard $(tablesSort:%=maps/VegX.%.csv))
vegxMaps += $(filter-out $(vegxMaps),$(wildcard maps/VegX.*.csv))
tables := $(vegxMaps:maps/VegX.%.csv=%)
vegbienMaps := $(subst VegX.,VegBIEN.,$(vegxMaps))

root := $(selfDir_3d1bc249)..
psqlAsBien := $(root)/bin/psql_vegbien

# Config
test_n ?= 2

#####

all: _always maps ;

.SUFFIXES:

_always:
.PHONY: _always

clean: _always
	$(RM) $(all)

remake: _always clean all ;

%.out: %.make _always
	./$* >$@
.PRECIOUS: %.out

$(root)/%: _always
	+$(subMake)

#####

reinstall: _always uninstall install ;

#####

maps: $(vegbienMaps) _always ;

all += $(vegbienMaps)

maps/VegBIEN.%.csv: maps/VegX.%.csv $(root)/mappings/VegX-VegBIEN.%.csv
	$(root)/bin/join_sort <$+ >$@
.PRECIOUS: maps/VegBIEN.%.csv

#####

import: _always $(addprefix import-,$(tables)) verify ;

log = $*$(if $(n),.n=$(n),).log

import-%: maps/VegBIEN.%.csv _always
	-$(import_cmd)
import_cmd = (set -x; "time" env commit=1 $(import) $<) \
$(if $(n),,>>$(log)) 2>&1$(if $(n),|tee -a $(log))

logs := $(wildcard *.log)

rm_logs: _always
	$(RM) $(logs)

#####

ifneq ($(wildcard verify.ref*),)

verify: verify.ref verify.out _always
	$(DIFF) $(+_)

all += verify.out

%.out: %.sql _always
	$(out_cmd)
.PRECIOUS: %.out
out_cmd = $(psqlAsBien) --no-align --field-separator='	' --pset=footer=off \
--pset=null=NULL <$< >$@

else
verify: _always ;
endif

#####

ifneq ($(wildcard test/),)

testOutputs := test/VegBIEN.xml test/import.out

test: _always $(testOutputs) ;

all += $(testOutputs)

define runTest
env test=1 n=$(test_n) $(1) $(+_)$(if $(2),, >$@) 2>&1$(if $(2), |\
grep -v -E '^$(2)$$' >$@) || { e=$$?; cat $@; exit $$e;}
$(DIFF) $@.ref $@
endef

test/VegBIEN.xml: $(vegbienMaps) _always
	$(call runTest,$(map),Processed .* input rows)
.PRECIOUS: test/VegBIEN.xml

test/import.out: $(vegbienMaps) _always
	$(call runTest,$(import))
.PRECIOUS: test/import.out

%-ok: _always
	$(CP) $* $*.ref

else
test: _always ;
endif

#####

# Each input type needs var $(mapEnv) and targets install, uninstall

dbFile := $(firstword $(wildcard src/*.sql))

ifneq ($(dbFile),)

dbFileBasename := $(basename $(notdir $(dbFile)))
dbEngineExt := $(subst .,,$(suffix $(dbFileBasename)))
db := $(basename $(dbFileBasename))

%.ref: %.ref.sql
	$(inputDbAsBien) $(db) <$< >$@
.PRECIOUS: %.ref

####

install: _always db ;

uninstall: _always rm_db ;

####

# Each DB engine needs vars $(dbEngine), $(dbAsBien) and targets db, rm_db

ifeq ($(dbEngineExt),my)

dbEngine := MySQL

bienPassword := $(shell cat $(root)/config/bien_password)
mysqlAs = mysql --user=$(1) --password='$(bienPassword)'
mysqlAsRoot := $(call mysqlAs,root)
dbAsBien := $(call mysqlAs,bien)

db: $(dbFile) _always
	-$(mysqlAsRoot) <$<
	echo "GRANT SELECT ON $(db).* TO 'bien'@'localhost';"|$(mysqlAsRoot)
# ignore errors in db import so that GRANT will still be run

rm_db: _always
	-echo "REVOKE ALL ON $(db).* FROM 'bien'@'localhost';"|$(mysqlAsRoot)
	echo "DROP DATABASE IF EXISTS $(db);"|$(mysqlAsRoot)
# ignore errors if grant not defined

###

else
$(error The DB filename $(dbFile) must have the form <db_name>.{my|pg}.sql)
endif

# Must come after dbEngine is set
mapEnv := in_engine=$(dbEngine) in_database=$(db)

####

else
mapEnv :=
install: _always ;
uninstall: _always ;
endif

#####

# Must come after mapEnv is set
map := $(if $(mapEnv),env $(mapEnv) ,)$(root)/map
import := env out_database=vegbien $(map)
